--
-- KroneZX
--
-- M@D Author:  Heady
-- M@D date: 23.11.2010
--
-- > Copyright (C) Heady - www.planet-ls.de < --
--

KroneZX = {};

function KroneZX.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Trailer, specializations);
end;

function KroneZX:load(xmlFile)

	self.updateSendEvent = SpecializationUtil.callSpecializationsFunction("updateSendEvent");

    local forageWgnSound = getXMLString(xmlFile, "vehicle.forageWgnSound#file");
    if forageWgnSound ~= nil and forageWgnSound ~= "" then
        forageWgnSound = Utils.getFilename(forageWgnSound, self.baseDirectory);
        self.forageWgnSound = createSample("forageWgnSound");
        self.forageWgnSoundEnabled = false;
        loadSample(self.forageWgnSound, forageWgnSound, false);
        self.forageWgnSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.forageWgnSound#pitchOffset"), 1);
        self.forageWgnSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.forageWgnSound#volume"), 1.0);
    end;
	
	local rollsCount = getXMLInt(xmlFile, "vehicle.rolls#count");
	self.rolls = {}
	if rollsCount ~= nil then		
		for i=1, rollsCount do
			local entry = {};
			local rollPartname = string.format("vehicle.rolls.roll" .. "%d", i);
			entry.index = Utils.indexToObject(self.components, getXMLString(xmlFile, rollPartname .. "#index"));
			entry.speedScale = Utils.getNoNil(getXMLInt(xmlFile, rollPartname .. "#speedScale"), 1)/1000;
			table.insert(self.rolls, entry);
		end;
	end;

    self.fillScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fillScale#value"), 1);

    self.isTurnedOn = false;
	
	self.pickupDown = false;
	self.currentRotLimit = {0,0,0};

end;

function KroneZX:delete()
    if self.forageWgnSound ~= nil then
        delete(self.forageWgnSound);
    end;
end;

function KroneZX:readStream(streamId, connection)
    self.isTurnedOn = streamReadBool(streamId);
	self.pickupDown = streamReadBool(streamId);
end;

function KroneZX:writeStream(streamId, connection)
    streamWriteBool(streamId, self.isTurnedOn);
	streamWriteBool(streamId, self.pickupDown);
end;

function KroneZX:readUpdateStream(streamId, timestamp, connection)
end;

function KroneZX:writeUpdateStream(streamId, connection, dirtyMask)
end;

function KroneZX:mouseEvent(posX, posY, isDown, isUp, button)
end;

function KroneZX:keyEvent(unicode, sym, modifier, isDown)
end;

function KroneZX:update(dt)

    if self:getIsActiveForInput() then
        if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA) then
            self.isTurnedOn = not self.isTurnedOn;
			self:updateSendEvent();
        end;
		if InputBinding.hasEvent(InputBinding.LOWER_IMPLEMENT) then
            self.pickupDown = not self.pickupDown;
			self:updateSendEvent();
        end;
    end;
end;

function KroneZX:updateTick(dt)

    if self:getIsActive() then	
	
        if self.isTurnedOn then

            if self.isServer and self.pickupDown and self.capacity > self.fillLevel then
				local cuttingAreasSend = {};
				for k, cuttingArea in pairs(self.cuttingAreas) do
					if self:getIsAreaActive(cuttingArea) then
						local x,y,z = getWorldTranslation(cuttingArea.start);
						local x1,y1,z1 = getWorldTranslation(cuttingArea.width);
						local x2,y2,z2 = getWorldTranslation(cuttingArea.height);

						table.insert(cuttingAreasSend, {x,z,x1,z1,x2,z2});
					end;
				end;
				if (table.getn(cuttingAreasSend) > 0) then				
					local area = 0;
					local fruitType = FruitUtil.FRUITTYPE_UNKNOWN;
					for fillType,v in pairs(self.fillTypes) do						
						if fillType ~= FruitUtil.FRUITTYPE_UNKNOWN then							
							if self.currentFillType == fillType or self.currentFillType == FruitUtil.FRUITTYPE_UNKNOWN then	
							
								fruitType = FruitUtil.fillTypeToFruitType[fillType];
								area = KroneZXAreaEvent.runLocally(cuttingAreasSend, fruitType);

								if area > 0 then
									self.currentFillType = fillType;
								end;
							end;
						end;
					end;
				
					if area > 0 then
						local pixelToSqm = g_currentMission:getFruitPixelsToSqm(); -- 4096px are mapped to 2048m
						local literPerSqm = g_strawLitersPerSqm * 2;
						local sqm = area*pixelToSqm;

						local deltaLevel = sqm*literPerSqm * self.fillScale;
						if fruitType == FruitUtil.FRUITTYPE_WHEAT or fruitType == FruitUtil.FRUITTYPE_BARLEY then
							deltaLevel = deltaLevel/2;
						end;

						self:setFillLevel(self.fillLevel+deltaLevel, self.currentFillType);
						g_server:broadcastEvent(KroneZXAreaEvent:new(cuttingAreasSend, fruitType));
					end;
				end;
            end;
			
			if self.rolls ~= nil then							
				rotate(self.rolls[1].index, dt*self.rolls[1].speedScale*self.attacherVehicle.lastSpeed*3600 , 0, 0);
				rotate(self.rolls[2].index, dt*self.rolls[2].speedScale , 0, 0);
			end;

            if self.isClient then
                if not self.forageWgnSoundEnabled and self:getIsActiveForSound() then
                    playSample(self.forageWgnSound, 0, self.forageWgnSoundVolume, 0);
                    setSamplePitch(self.forageWgnSound, self.forageWgnSoundPitchOffset);
                    self.forageWgnSoundEnabled = true;
                end;
            end;
        end;
		
		if self.isServer then
			local newRotLimit = {0,0,0};
			newRotLimit = Utils.getMovedLimitedValues(self.currentRotLimit, {0,0,0}, {30,0,0}, 3, 1000, dt, self.pickupDown);
			if math.abs(newRotLimit[1] - self.currentRotLimit[1]) > 0.001 then
				setJointRotationLimit(self.componentJoints[3].jointIndex, 0, true, Utils.degToRad(0), Utils.degToRad(newRotLimit[1]));
			end;			
			self.currentRotLimit = newRotLimit;	
		end;
		
        if self.isClient then
            if self.forageWgnSoundEnabled and not self.isTurnedOn then
                stopSample(self.forageWgnSound);
                self.forageWgnSoundEnabled = false;
            end;
        end;
		
    end;
end;

function KroneZX:draw()

	if self.pickupDown then
		g_currentMission:addHelpButtonText(string.format(g_i18n:getText("KRONEZX_PICKUPUP"), self.typeDesc), InputBinding.LOWER_IMPLEMENT);
    else
        g_currentMission:addHelpButtonText(string.format(g_i18n:getText("KRONEZX_PICKUPDOWN"), self.typeDesc), InputBinding.LOWER_IMPLEMENT);
    end;
    if self.isTurnedOn then
        g_currentMission:addHelpButtonText(string.format(g_i18n:getText("turn_off_OBJECT"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA);
    else
        g_currentMission:addHelpButtonText(string.format(g_i18n:getText("turn_on_OBJECT"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA);
    end;
	
end;

function KroneZX:onDetach()
    if self.deactivateOnDetach then
        KroneZX.onDeactivate(self);
    else
        KroneZX.onDeactivateSounds(self);
    end;
end;

function KroneZX:onLeave()
    if self.deactivateOnLeave then
        KroneZX.onDeactivate(self);
    else
        KroneZX.onDeactivateSounds(self);
    end;
end;

function KroneZX:onDeactivate()
    self.isTurnedOn = false;
    KroneZX.onDeactivateSounds(self);
end;


function KroneZX:onDeactivateSounds()
    if self.forageWgnSoundEnabled then
        stopSample(self.forageWgnSound);
        self.forageWgnSoundEnabled = false;
    end;
end;

function KroneZX:updateSendEvent()	

	if g_server ~= nil then
		g_server:broadcastEvent(MPEvent:new(self));
	else
		g_client:getServerConnection():sendEvent(MPEvent:new(self));
	end;

end;